package cat.hack3.codingtests.marsrover;

import java.util.logging.Logger;

/**
 * Notice that the representation is the following:
 * Latitude is Y, starts from 1 on the top and goes down incrementally
 * Longitude is X, starts from 1 on the left and goes to right incrementally
 *     1    2    3
 * 1  1-1  1-2  1-3
 * 2  2-1  2-2  2-3
 * 3  3-1  3-2  3-3
 */
public class MarsMap {
    Logger logger = Logger.getLogger(this.getClass().getName());

    private final int width;
    private final int height;
    private Coordinates currentPosition;

    public MarsMap(int width, int height, Coordinates startingCoordinates) {
        this.width = width;
        this.height = height;
        currentPosition = startingCoordinates;
    }

    public Coordinates updatePositionTowards(Direction direction) {
        return currentPosition = resolveNextPositionTowards(direction);
    }

    private Coordinates resolveNextPositionTowards(Direction direction) {
        var newLatitude = currentPosition.getLatitude();
        var newLongitude = currentPosition.getLongitude();
        switch (direction) {
            case NORTH -> newLatitude--;
            case SOUTH -> newLatitude++;
            case WEST -> newLongitude--;
            case EAST -> newLongitude++;
        }
        logger.info(String.format("Updated coordinates towards %s, now is %d-%d",
                direction, newLatitude, newLongitude));

        return new Coordinates(newLatitude, newLongitude);
    }
}
